# This file is generated by Lemonldap::NG::Manager::Build. Don't modify it by hand
package Lemonldap::NG::Common::Conf::DefaultValues;

our $VERSION = '2.20.0';

sub defaultValues {
    return {
        'activeTimer'        => 1,
        'ADPwdExpireWarning' => 0,
        'ADPwdMaxAge'        => 0,
        'apacheAuthnLevel'   => 3,
        'applicationList'    => {
            'default' => {
                'catname' => 'Default category',
                'type'    => 'category'
            }
        },
        'authChoiceParam' => 'lmAuth',
        'authentication'  => 'Demo',
        'available2F'     =>
          'TOTP,REST,Mail2F,Ext2F,WebAuthn,Yubikey,Radius,Password,Okta',
        'available2FSelfRegistration'     => 'Password,TOTP,WebAuthn,Yubikey',
        'bruteForceProtectionLockTimes'   => '15, 30, 60, 300, 600',
        'bruteForceProtectionMaxAge'      => 300,
        'bruteForceProtectionMaxFailed'   => 3,
        'bruteForceProtectionMaxLockTime' => 900,
        'bruteForceProtectionTempo'       => 30,
        'captcha_login_enabled'           => 0,
        'captcha_mail_enabled'            => 1,
        'captcha_register_enabled'        => 1,
        'captcha_size'                    => 6,
        'casAccessControlPolicy'          => 'none',
        'casAuthnLevel'                   => 1,
        'casTicketExpiration'             => 0,
        'certificateResetByMailCeaAttribute'         => 'description',
        'certificateResetByMailCertificateAttribute' =>
          'userCertificate;binary',
        'certificateResetByMailValidityDelay' => 0,
        'checkDevOpsCheckSessionAttributes'   => 1,
        'checkDevOpsDisplayNormalizedHeaders' => 1,
        'checkDevOpsDownload'                 => 1,
        'checkHIBPRequired'                   => 1,
        'checkHIBPURL' => 'https://api.pwnedpasswords.com/range/',
        'checkMsg'     => 5,
        'checkTime'    => 600,
        'checkUserDisplayComputedSession'   => 1,
        'checkUserDisplayEmptyHeaders'      => 0,
        'checkUserDisplayEmptyValues'       => 0,
        'checkUserDisplayHiddenAttributes'  => 0,
        'checkUserDisplayHistory'           => 0,
        'checkUserDisplayNormalizedHeaders' => 0,
        'checkUserDisplayPersistentInfo'    => 0,
        'checkUserHiddenAttributes' => '_loginHistory, _session_id, hGroups',
        'checkUserIdRule'           => 1,
        'checkXSS'                  => 1,
        'confirmFormMethod'         => 'post',
        'contextSwitchingIdRule'    => 1,
        'contextSwitchingPrefix'    => 'switching',
        'contextSwitchingRule'      => 0,
        'contextSwitchingStopWithLogout' => 1,
        'cookieName'                     => 'lemonldap',
        'corsAllow_Credentials'          => 'true',
        'corsAllow_Headers'              => '*',
        'corsAllow_Methods'              => 'POST,GET',
        'corsAllow_Origin'               => '*',
        'corsEnabled'                    => 1,
        'corsExpose_Headers'             => '*',
        'corsMax_Age'                    => '86400',
        'crowdsecAction'                 => 'reject',
        'cspConnect'                     => '\'self\'',
        'cspDefault'                     => '\'self\'',
        'cspFont'                        => '\'self\'',
        'cspFormAction'                  => '*',
        'cspFrameAncestors'              => '',
        'cspImg'                         => '\'self\' data:',
        'cspScript'                      => '\'self\'',
        'cspStyle'                       => '\'self\'',
        'dbiAuthnLevel'                  => 2,
        'dbiExportedVars'                => {},
        'decryptValueRule'               => 0,
        'defaultNewKeySize'              => 2048,
        'demoExportedVars'               => {
            'cn'   => 'cn',
            'mail' => 'mail',
            'uid'  => 'uid'
        },
        'displaySessionId' => 1,
        'domain'           => 'example.com',
        'eventQueueName'   => 'llng_events',
        'exportedVars'     => {
            'UA' => 'HTTP_USER_AGENT'
        },
        'ext2fActivation'      => 0,
        'ext2fCodeActivation'  => '\\d{6}',
        'facebookAuthnLevel'   => 1,
        'facebookExportedVars' => {},
        'facebookUserField'    => 'id',
        'failedLoginNumber'    => 5,
        'findUserControl'      => '^[*\\w]+$',
        'findUserWildcard'     => '*',
        'formTimeout'          => 120,
        'githubAuthnLevel'     => 1,
        'githubScope'          => 'user:email',
        'githubUserField'      => 'login',
        'globalLogoutRule'     => 0,
        'globalLogoutTimer'    => 1,
        'globalStorage'        => 'Apache::Session::File',
        'globalStorageOptions' => {
            'Directory'      => '/var/lib/lemonldap-ng/sessions/',
            'generateModule' =>
              'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
            'LockDirectory' => '/var/lib/lemonldap-ng/sessions/lock/'
        },
        'gpgAuthnLevel'                 => 5,
        'gpgDb'                         => '',
        'grantSessionRules'             => {},
        'groups'                        => {},
        'handlerInternalCache'          => 15,
        'handlerServiceTokenTTL'        => 30,
        'hiddenAttributes'              => '_password _2fDevices',
        'hideOldPassword'               => 1,
        'httpOnly'                      => 1,
        'https'                         => -1,
        'impersonationHiddenAttributes' => '_2fDevices, _loginHistory',
        'impersonationIdRule'           => 1,
        'impersonationMergeSSOgroups'   => 0,
        'impersonationPrefix'           => 'real_',
        'impersonationRule'             => 0,
        'impersonationSkipEmptyValues'  => 1,
        'infoFormMethod'                => 'get',
        'issuerDBCASPath'               => '^/cas/',
        'issuerDBCASRule'               => 1,
        'issuerDBGetParameters'         => {},
        'issuerDBGetPath'               => '^/get/',
        'issuerDBGetRule'               => 1,
        'issuerDBJitsiMeetTokensPath'   => '^/jitsi/',
        'issuerDBJitsiMeetTokensRule'   => 1,
        'issuerDBOpenIDConnectPath'     => '^/oauth2/',
        'issuerDBOpenIDConnectRule'     => 1,
        'issuerDBOpenIDPath'            => '^/openidserver/',
        'issuerDBOpenIDRule'            => 1,
        'issuerDBSAMLPath'              => '^/saml/',
        'issuerDBSAMLRule'              => 1,
        'issuersTimeout'                => 120,
        'jitsiExpiration'               => '300',
        'jitsiSigningAlg'               => 'RS256',
        'jsRedirect'                    => 0,
        'krbAuthnLevel'                 => 3,
        'krbRemoveDomain'               => 1,
        'ldapAuthnLevel'                => 2,
        'ldapBase'                      => 'dc=example,dc=com',
        'ldapExportedVars'              => {
            'cn'   => 'cn',
            'mail' => 'mail',
            'uid'  => 'uid'
        },
        'ldapGroupAttributeName'          => 'member',
        'ldapGroupAttributeNameGroup'     => 'dn',
        'ldapGroupAttributeNameSearch'    => 'cn',
        'ldapGroupAttributeNameUser'      => 'dn',
        'ldapGroupObjectClass'            => 'groupOfNames',
        'ldapIOTimeout'                   => 10,
        'ldapPasswordResetAttribute'      => 'pwdReset',
        'ldapPasswordResetAttributeValue' => 'TRUE',
        'ldapPwdEnc'                      => 'utf-8',
        'ldapSearchDeref'                 => 'find',
        'ldapServer'                      => 'ldap://localhost',
        'ldapTimeout'                     => 10,
        'ldapUsePasswordResetAttribute'   => 1,
        'ldapVerify'                      => 'require',
        'ldapVersion'                     => 3,
        'linkedInAuthnLevel'              => 1,
        'linkedInFields'             => 'id,first-name,last-name,email-address',
        'linkedInScope'              => 'r_liteprofile r_emailaddress',
        'linkedInUserField'          => 'emailAddress',
        'localSessionStorage'        => 'Cache::FileCache',
        'localSessionStorageOptions' => {
            'cache_depth'        => 3,
            'cache_root'         => '/var/cache/lemonldap-ng',
            'default_expires_in' => 600,
            'directory_umask'    => '007',
            'namespace'          => 'lemonldap-ng-sessions'
        },
        'locationDetectGeoIpLanguages' => 'en, fr',
        'locationRules'                => {
            'default' => 'deny'
        },
        'logoutServices' => {},
        'lwpOpts'        => {
            'timeout' => 10
        },
        'macros'                    => {},
        'mail2fActivation'          => 0,
        'mail2fCodeRegex'           => '\\d{6}',
        'mailCharset'               => 'utf-8',
        'mailFrom'                  => 'noreply@example.com',
        'mailSessionKey'            => 'mail',
        'mailTimeout'               => 0,
        'managerDn'                 => '',
        'managerPassword'           => '',
        'max2FDevices'              => 10,
        'max2FDevicesNameLength'    => 20,
        'messageBrokerOptions'      => {},
        'multiValuesSeparator'      => '; ',
        'mySessionAuthorizedRWKeys' => [
            '_appsListOrder',      '_oidcConnectedRP',
            '_oidcConnectedRPIDs', '_oidcConsents'
        ],
        'newLocationWarningLocationAttribute'        => 'ipAddr',
        'newLocationWarningLocationDisplayAttribute' => '',
        'newLocationWarningMaxValues'                => '0',
        'notificationDefaultCond'                    => '',
        'notificationServerPOST'                     => 1,
        'notificationServerSentAttributes'           =>
          'uid reference date title subtitle text check',
        'notificationsMaxRetrieve'   => 3,
        'notificationStorage'        => 'File',
        'notificationStorageOptions' => {
            'dirName' => '/var/lib/lemonldap-ng/notifications'
        },
        'notificationWildcard'                   => 'allusers',
        'notifyDeleted'                          => 1,
        'nullAuthnLevel'                         => 0,
        'oidcAuthnLevel'                         => 1,
        'oidcRPCallbackGetParam'                 => 'openidconnectcallback',
        'oidcRPStateTimeout'                     => 600,
        'oidcServiceAccessTokenExpiration'       => 3600,
        'oidcServiceAllowAuthorizationCodeFlow'  => 1,
        'oidcServiceAuthorizationCodeExpiration' => 60,
        'oidcServiceEncAlgorithmAlg'             => 'RSA-OAEP',
        'oidcServiceEncAlgorithmEnc'             => 'A256GCM',
        'oidcServiceIDTokenExpiration'           => 3600,
        'oidcServiceKeyTypeEnc'                  => 'RSA',
        'oidcServiceKeyTypeSig'                  => 'RSA',
        'oidcServiceMetaDataAuthnContext'        => {
            'loa-1' => 1,
            'loa-2' => 2,
            'loa-3' => 3,
            'loa-4' => 4,
            'loa-5' => 5
        },
        'oidcServiceMetaDataAuthorizeURI'     => 'authorize',
        'oidcServiceMetaDataBackChannelURI'   => 'blogout',
        'oidcServiceMetaDataCheckSessionURI'  => 'checksession.html',
        'oidcServiceMetaDataEndSessionURI'    => 'logout',
        'oidcServiceMetaDataFrontChannelURI'  => 'flogout',
        'oidcServiceMetaDataIntrospectionURI' => 'introspect',
        'oidcServiceMetaDataJWKSURI'          => 'jwks',
        'oidcServiceMetaDataRegistrationURI'  => 'register',
        'oidcServiceMetaDataTokenURI'         => 'token',
        'oidcServiceMetaDataUserInfoURI'      => 'userinfo',
        'oidcServiceNewKeyTypeSig'            => 'RSA',
        'oidcServiceOfflineSessionExpiration' => 2592000,
        'oidcServiceOldKeyTypeEnc'            => 'RSA',
        'oidcServiceOldKeyTypeSig'            => 'RSA',
        'okta2fActivation'                    => 0,
        'openIdAuthnLevel'                    => 1,
        'openIdExportedVars'                  => {},
        'openIdIDPList'                       => '0;',
        'openIdSPList'                        => '0;',
        'openIdSreg_email'                    => 'mail',
        'openIdSreg_fullname'                 => 'cn',
        'openIdSreg_nickname'                 => 'uid',
        'openIdSreg_timezone'                 => '_timezone',
        'pamAuthnLevel'                       => 2,
        'pamService'                          => 'login',
        'password2fActivation'                => 0,
        'password2fSelfRegistration'          => 0,
        'password2fUserCanRemoveKey'          => 1,
        'passwordDB'                          => 'Demo',
        'passwordPolicyActivation'            => 1,
        'passwordPolicyMaxSize'               => 0,
        'passwordPolicyMinDigit'              => 0,
        'passwordPolicyMinLower'              => 0,
        'passwordPolicyMinSize'               => 0,
        'passwordPolicyMinSpeChar'            => 0,
        'passwordPolicyMinUpper'              => 0,
        'passwordPolicySpecialChar'           => '__ALL__',
        'passwordResetAllowedRetries'         => 3,
        'persistentSessionAttributes'         =>
          '_loginHistory _2fDevices _oidcConsents notification_',
        'port'                          => -1,
        'portal'                        => 'http://auth.example.com/',
        'portalAntiFrame'               => 1,
        'portalCheckLogins'             => 1,
        'portalDisplayAppslist'         => 1,
        'portalDisplayChangePassword'   => '$_auth =~ /^(LDAP|DBI|Demo)$/',
        'portalDisplayGeneratePassword' => 1,
        'portalDisplayLoginHistory'     => 1,
        'portalDisplayLogout'           => 1,
        'portalDisplayOidcConsents'     =>
          '$_oidcConsents && $_oidcConsents =~ /\\w+/',
        'portalDisplayOrder' =>
          'Appslist ChangePassword LoginHistory OidcConsents Logout',
        'portalDisplayRefreshMyRights' => 1,
        'portalDisplayRegister'        => 1,
        'portalErrorOnExpiredSession'  => 1,
        'portalFavicon'                => 'common/favicon.ico',
        'portalForceAuthnInterval'     => 300,
        'portalMainLogo'               => 'common/logos/logo_llng_400px.png',
        'portalPingInterval'           => 60000,
        'portalRequireOldPassword'     => 1,
        'portalSkin'                   => 'bootstrap',
        'portalUserAttr'               => '_user',
        'proxyAuthnLevel'              => 2,
        'proxyAuthServiceChoiceParam'  => 'lmAuth',
        'radius2fActivation'           => 0,
        'radius2fRequestAttributes'    => {},
        'radius2fTimeout'              => 20,
        'radiusAuthnLevel'             => 3,
        'radiusExportedVars'           => {},
        'radiusRequestAttributes'      => {},
        'randomPasswordRegexp'         => '[A-Z]{3}[a-z]{5}.\\d{2}',
        'redirectFormMethod'           => 'get',
        'registerDB'                   => 'Null',
        'registerTimeout'              => 0,
        'reloadTimeout'                => 5,
        'rememberAuthChoiceRule'       => 0,
        'rememberCookieName'           => 'llngrememberauthchoice',
        'rememberCookieTimeout'        => 31536000,
        'rememberTimer'                => 5,
        'remoteGlobalStorage' => 'Lemonldap::NG::Common::Apache::Session::SOAP',
        'remoteGlobalStorageOptions' => {
            'ns' =>
              'http://auth.example.com/Lemonldap/NG/Common/PSGI/SOAPService',
            'proxy' => 'http://auth.example.com/sessions'
        },
        'requireToken'                                         => 1,
        'rest2fActivation'                                     => 0,
        'restAuthnLevel'                                       => 2,
        'restClockTolerance'                                   => 15,
        'sameSite'                                             => '',
        'samlAttributeAuthorityDescriptorAttributeServiceSOAP' =>
          'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/AA/SOAP;',
        'samlAuthnContextMapKerberos'                   => 4,
        'samlAuthnContextMapPassword'                   => 2,
        'samlAuthnContextMapPasswordProtectedTransport' => 3,
        'samlAuthnContextMapTLSClient'                  => 5,
        'samlEntityID' => '#PORTAL#/saml/metadata',
        'samlIDPSSODescriptorArtifactResolutionServiceArtifact' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/artifact',
        'samlIDPSSODescriptorSingleLogoutServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/singleLogout;#PORTAL#/saml/singleLogoutReturn',
        'samlIDPSSODescriptorSingleLogoutServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/singleLogout;#PORTAL#/saml/singleLogoutReturn',
        'samlIDPSSODescriptorSingleLogoutServiceSOAP' =>
'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/singleLogoutSOAP;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPArtifact' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact;#PORTAL#/saml/singleSignOnArtifact;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/singleSignOn;',
        'samlIDPSSODescriptorSingleSignOnServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/singleSignOn;',
        'samlIDPSSODescriptorWantAuthnRequestsSigned' => 1,
        'samlMetadataForceUTF8'                       => 1,
        'samlNameIDFormatMapEmail'                    => 'mail',
        'samlNameIDFormatMapKerberos'                 => 'uid',
        'samlNameIDFormatMapWindows'                  => 'uid',
        'samlNameIDFormatMapX509'                     => 'mail',
        'samlOrganizationDisplayName'                 => 'Example',
        'samlOrganizationName'                        => 'Example',
        'samlOrganizationURL'        => 'http://www.example.com',
        'samlOverrideIDPEntityID'    => '',
        'samlRelayStateTimeout'      => 600,
        'samlServiceSignatureMethod' => 'RSA_SHA256',
        'samlSPSSODescriptorArtifactResolutionServiceArtifact' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/artifact',
        'samlSPSSODescriptorAssertionConsumerServiceHTTPArtifact' =>
'0;1;urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact;#PORTAL#/saml/proxySingleSignOnArtifact',
        'samlSPSSODescriptorAssertionConsumerServiceHTTPPost' =>
'1;0;urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/proxySingleSignOnPost',
        'samlSPSSODescriptorAuthnRequestsSigned'         => 1,
        'samlSPSSODescriptorSingleLogoutServiceHTTPPost' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST;#PORTAL#/saml/proxySingleLogout;#PORTAL#/saml/proxySingleLogoutReturn',
        'samlSPSSODescriptorSingleLogoutServiceHTTPRedirect' =>
'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect;#PORTAL#/saml/proxySingleLogout;#PORTAL#/saml/proxySingleLogoutReturn',
        'samlSPSSODescriptorSingleLogoutServiceSOAP' =>
'urn:oasis:names:tc:SAML:2.0:bindings:SOAP;#PORTAL#/saml/proxySingleLogoutSOAP;',
        'samlSPSSODescriptorWantAssertionsSigned' => 1,
        'scrollTop'                               => 400,
        'securedCookie'                           => 0,
        'sfEngine'                                => '::2F::Engines::Default',
        'sfManagerRule'                           => 1,
        'sfRemovedMsgRule'                        => 0,
        'sfRemovedNotifMsg'                       =>
'_removedSF_ expired second factor(s) has/have been removed (_nameSF_)!',
        'sfRemovedNotifRef'       => 'RemoveSF',
        'sfRemovedNotifTitle'     => 'Second factor notification',
        'sfRequired'              => 0,
        'showLanguages'           => 1,
        'singleIP'                => 0,
        'singleSession'           => 0,
        'singleUserByIP'          => 0,
        'slaveAuthnLevel'         => 2,
        'slaveExportedVars'       => {},
        'SMTPServer'              => '',
        'SMTPTLS'                 => '',
        'soapProxyUrn'            => 'urn:Lemonldap/NG/Common/PSGI/SOAPService',
        'SSLAuthnLevel'           => 5,
        'SSLIssuerVar'            => 'SSL_CLIENT_I_DN',
        'SSLVar'                  => 'SSL_CLIENT_S_DN_Email',
        'SSLVarIf'                => {},
        'statusQueueName'         => 'llng_status',
        'stayConnected'           => 0,
        'stayConnectedCookieName' => 'llngconnection',
        'stayConnectedTimeout'    => 2592000,
        'successLoginNumber'      => 5,
        'timeout'                 => 72000,
        'timeoutActivity'         => 0,
        'timeoutActivityInterval' => 60,
        'totp2fActivation'        => 0,
        'totp2fDigits'            => 6,
        'totp2fInterval'          => 30,
        'totp2fRange'             => 1,
        'totp2fSelfRegistration'  => 0,
        'totp2fUserCanRemoveKey'  => 1,
        'trustedBrowserRule'      => 0,
        'trustedBrowserUseTotp'   => 1,
        'twitterAuthnLevel'       => 1,
        'twitterUserField'        => 'screen_name',
        'upgradeSession'          => 1,
        'userControl'             => '^[\\w\\.\\-@]+$',
        'userDB'                  => 'Same',
        'useRedirectAjaxOnUnauthorized' => 1,
        'useRedirectOnError'            => 1,
        'useSafeJail'                   => 1,
        'viewerHiddenKeys'      => 'samlIDPMetaDataNodes, samlSPMetaDataNodes',
        'webauthn2fActivation'  => 0,
        'webauthn2fAttestation' => 'none',
        'webauthn2fSelfRegistration' => 0,
        'webauthn2fUserCanRemoveKey' => 1,
        'webauthn2fUserVerification' => 'preferred',
        'webauthnAppId'              => 1,
        'webauthnAuthnLevel'         => 3,
        'webIDAuthnLevel'            => 1,
        'webIDExportedVars'          => {},
        'whatToTrace'                => 'uid',
        'yubikey2fActivation'        => 0,
        'yubikey2fPublicIDSize'      => 12,
        'yubikey2fSelfRegistration'  => 0,
        'yubikey2fUserCanRemoveKey'  => 1
    };
}

1;
